@@ -30,6 +30,7 @@
 #include <language/duchain/persistentsymboltable.h>
 #include <language/duchain/types/functiontype.h>
 #include <language/duchain/types/integraltype.h>
+#include <language/duchain/types/structuretype.h>
 
 #define ifDebug(x)
 
@@ -41,7 +42,8 @@ namespace Php
 ExpressionVisitor::ExpressionVisitor(EditorIntegrator* editor)
         : m_editor(editor), m_createProblems(false),
           m_offset(CursorInRevision::invalid()), m_currentContext(0),
-          m_isAssignmentExpressionEqual(false)
+          m_isAssignmentExpressionEqual(false),
+          m_inDefine(false)
 {
 }
 
@@ -61,30 +63,25 @@ DeclarationPointer ExpressionVisitor::processVariable(VariableIdentifierAst* var
 
     ifDebug(kDebug() << "processing variable" << identifier.toString() << position.castToSimpleCursor();)
 
+    DUChainReadLocker lock;
+
     if (identifier.nameEquals(Identifier("this"))) {
-        DUChainReadLocker lock(DUChain::lock());
         if (m_currentContext->parentContext()
                 && m_currentContext->parentContext()->type() == DUContext::Class
                 && m_currentContext->parentContext()->owner()) {
             ret = m_currentContext->parentContext()->owner();
         }
     } else {
-        DUChainReadLocker lock(DUChain::lock());
         //DontSearchInParent-flag because (1) in Php global variables aren't available in function
         //context and (2) a function body consists of a single context (so this is no problem)
-        QList<Declaration*> decls = m_currentContext->findDeclarations(identifier, position,
-                                                            0, DUContext::DontSearchInParent);
-        for (int i = decls.count() - 1; i >= 0; i--) {
-            Declaration *dec = decls.at(i);
-            if (dec->kind() == Declaration::Instance && dynamic_cast<VariableDeclaration*>(dec)) {
-                ret = dec;
-                break;
-            }
-        }
+        ret = findVariableDeclaration(m_currentContext, identifier, position, DUContext::DontSearchInParent);
+    }
+    if (!ret && m_currentContext->type() == DUContext::Namespace)
+    {
+        ret = findVariableDeclaration(m_currentContext, identifier, position, DUContext::NoSearchFlags);
     }
     if (!ret) {
         //look for a function argument
-        DUChainReadLocker lock(DUChain::lock());
         ///TODO: why doesn't m_currentContext->findDeclarations() work?
         ///      evaluate if the stuff below is fast enough (faster?) than findDeclarations()
         ///see r1028306
@@ -104,7 +101,6 @@ DeclarationPointer ExpressionVisitor::processVariable(VariableIdentifierAst* var
     }
     if (!ret) {
         //look for a superglobal variable
-        DUChainReadLocker lock(DUChain::lock());
         foreach(Declaration* dec, m_currentContext->topContext()->findDeclarations(identifier, position)) {
             VariableDeclaration* varDec = dynamic_cast<VariableDeclaration*>(dec);
             if (varDec && varDec->isSuperglobal()) {
@@ -113,6 +109,9 @@ DeclarationPointer ExpressionVisitor::processVariable(VariableIdentifierAst* var
             }
         }
     }
+
+    lock.unlock();
+
     if ( !m_isAssignmentExpressionEqual || identifier.nameEquals( Identifier("this") )
          // might be something like $s = $s . $s; in which case we have to add a use for the first $s
          || (ret && ret->range().end < position) )
@@ -174,7 +173,8 @@ void ExpressionVisitor::visitVariable(VariableAst* node)
     if ( node->variablePropertiesSequence &&
          node->variablePropertiesSequence->front() &&
          node->variablePropertiesSequence->front()->element &&
-         node->variablePropertiesSequence->front()->element->objectProperty ) {
+         node->variablePropertiesSequence->front()->element->variableProperty &&
+         node->variablePropertiesSequence->front()->element->variableProperty->objectProperty ) {
         // make sure we mark $foo as a use in $foo->...
         bool isAssignmentExpressionEqual = m_isAssignmentExpressionEqual;
         m_isAssignmentExpressionEqual = false;
@@ -287,9 +287,29 @@ void ExpressionVisitor::visitFunctionCallParameterList( FunctionCallParameterLis
     m_result.setType(type);
 }
 
+void ExpressionVisitor::visitFunctionCallParameterListElement(FunctionCallParameterListElementAst* node)
+{
+    DefaultVisitor::visitFunctionCallParameterListElement(node);
+
+    if (m_inDefine) m_inDefine = false; //reset after first parameter passed, the second argument can be a class name
+}
+
 void ExpressionVisitor::visitFunctionCall(FunctionCallAst* node)
 {
+    if (node->stringFunctionNameOrClass && !node->stringFunctionName && !node->varFunctionName) {
+        QualifiedIdentifier id = identifierForNamespace(node->stringFunctionNameOrClass, m_editor);
+        if (id.toString(true) == "define"
+                && node->stringParameterList && node->stringParameterList->parametersSequence
+                && node->stringParameterList->parametersSequence->count() > 0) {
+            //in a define() call the first argument is the constant name. we don't want to look for a class name to build uses
+            m_inDefine = true;
+        }
+    }
+
     DefaultVisitor::visitFunctionCall(node);
+
+    m_inDefine = false;
+
     if (node->stringFunctionNameOrClass) {
         if (node->stringFunctionName) {
             //static function call foo::bar()
@@ -436,8 +456,24 @@ void ExpressionVisitor::visitScalar(ScalarAst *node)
     } else if (node->encapsList) {
         m_result.setType(AbstractType::Ptr(new IntegralType(IntegralType::TypeString)));
     }
+
+    if (!m_inDefine && node->commonScalar && node->commonScalar->scalarType == ScalarTypeString) {
+        QString str = m_editor->parseSession()->symbol(node->commonScalar);
+        QRegExp exp("^['\"]([A-Za-z0-9_]+)['\"]$");
+        if (exp.exactMatch(str)) {
+            //that *could* be a class name
+            QualifiedIdentifier id(exp.cap(1).toLower());
+            DeclarationPointer declaration = findDeclarationImport(ClassDeclarationType, id);
+            if (declaration) {
+                usingDeclaration(node->commonScalar, declaration);
+            } else {
+                m_result.setHadUnresolvedIdentifiers(true);
+            }
+        }
+    }
 }
 
+
 void ExpressionVisitor::visitStaticScalar(StaticScalarAst *node)
 {
     if (node->ducontext) {
@@ -747,4 +783,19 @@ DeclarationPointer ExpressionVisitor::findDeclarationImport( DeclarationType dec
     return findDeclarationImportHelper(m_currentContext, identifier, declarationType);
 }
 
+Declaration* ExpressionVisitor::findVariableDeclaration(DUContext* context, Identifier identifier,
+                                                        CursorInRevision position, DUContext::SearchFlag flag)
+{
+    QList<Declaration*> decls = context->findDeclarations(identifier, position,
+                                                        0, flag);
+    for (int i = decls.count() - 1; i >= 0; i--) {
+        Declaration *dec = decls.at(i);
+        if (dec->kind() == Declaration::Instance && dynamic_cast<VariableDeclaration*>(dec)) {
+            return dec;
+        }
+    }
+
+    return NULL;
+}
+
 }
